rm(list=ls())

################################################################################
#1.GET PACKAGES
################################################################################
library(dplyr)
library(tidyr)
library(ggplot2)
library(RColorBrewer)

################################################################################
#2.CREATE DATA (FOR PURPOSE OF THE EXAMPLE ONLY!)
################################################################################
#Example with 7 fictive subjects and 3 states###################################
#prepare vectors for birth and state
birth <- c(1988.25,1990.5,1992.25,1994,1995.75,1998,1999.5)
S <- c("Alone","Cohabiting","Married")

#observation 1
start <- c(2005,2010.5,2017.5)
State <- c(S[1],S[2],S[3])
s1 <- data.frame(id=rep(1,length(start)),birth=rep(birth[1],length(start)),start=start,stop=c(start[2:length(start)],2019),State=State)

#observation 2
start <- c(2007,2010,2016.25,2017.5)
State <- c(S[1],S[2],S[1],S[1])
s2 <- data.frame(id=rep(2,length(start)),birth=rep(birth[2],length(start)),start=start,stop=c(start[2:length(start)],2019),State=State)

#observation 3
start <- c(2009,2009.5,2010.25,2012.75,2014,2016.5,2018)
State <- c(S[1],S[2],S[1],S[2],S[1],S[2],S[1])
s3 <- data.frame(id=rep(3,length(start)),birth=rep(birth[3],length(start)),start=start,stop=c(start[2:length(start)],2019),State=State)

#observation 4
start <- c(2011,2012.75)
State <- c(S[1],S[3])
s4 <- data.frame(id=rep(4,length(start)),birth=rep(birth[4],length(start)),start=start,stop=c(start[2:length(start)],2019),State=State)

#observation 5
start <- c(2013,2014.25,2015.5,2016,2018.5)
State <- c(S[2],S[1],S[2],S[3],S[1])
s5 <- data.frame(id=rep(5,length(start)),birth=rep(birth[5],length(start)),start=start,stop=c(start[2:length(start)],2019),State=State)

#observation 6
start <- c(2015)
State <- c(S[1])
s6 <- data.frame(id=rep(6,length(start)),birth=rep(birth[6],length(start)),start=start,stop=2019,State=State)

#observation 7
start <- c(2017,2017.75,2018.25)
State <- c(S[1],S[2],S[1])
s7 <- data.frame(id=rep(7,length(start)),birth=rep(birth[7],length(start)),start=start,stop=c(start[2:length(start)],2019),State=State)

s <- list(s1,s2,s3,s4,s5,s6,s7)

df <- bind_rows(s)

remove(s1,s2,s3,s4,s5,s6,s7,s)

################################################################################
#3. VISUALIZE DATA
################################################################################
ggplot(df)+
  geom_segment(aes(x=start,xend=stop,y=start-birth,yend=stop-birth,color=State,group=State),size=4)+
  scale_y_continuous(breaks=seq(16,32,2),limits=c(16,32))+
  scale_x_continuous(breaks=seq(2005,2019,2),limits=c(2005,2019))+
  scale_color_brewer(palette="RdYlGn")+
  theme_bw()+
  theme(panel.grid.minor.x = element_blank(),panel.grid.minor.y = element_blank(),
        legend.position="bottom")+
  xlab("Time (t)")+
  ylab("Age (a)")+
  geom_segment(aes(x=2005,xend=2005,y=16,yend=18))+
  geom_segment(aes(x=2007,xend=2007,y=16,yend=20))+
  geom_segment(aes(x=2009,xend=2009,y=16,yend=22))+
  geom_segment(aes(x=2011,xend=2011,y=16,yend=24))+
  geom_segment(aes(x=2013,xend=2013,y=16,yend=26))+
  geom_segment(aes(x=2015,xend=2015,y=16,yend=28))+
  geom_segment(aes(x=2017,xend=2017,y=16,yend=30))+
  geom_segment(aes(x=2019,xend=2019,y=18,yend=32))+
  geom_segment(aes(x=2005,xend=2019,y=18,yend=32))+
  geom_segment(aes(x=2005,xend=2019,y=16,yend=30))+
  geom_segment(aes(x=2007,xend=2019,y=16,yend=28))+
  geom_segment(aes(x=2009,xend=2019,y=16,yend=26))+
  geom_segment(aes(x=2011,xend=2019,y=16,yend=24))+
  geom_segment(aes(x=2013,xend=2019,y=16,yend=22))+
  geom_segment(aes(x=2015,xend=2019,y=16,yend=20))+
  geom_segment(aes(x=2017,xend=2019,y=16,yend=18))

################################################################################
#4. CALCULATIONS
#In this specific case values correspond to unique observations
#in practice, they would represent averages among several people with same year(s) of birth
################################################################################
#set parameters
#minimum age from which we start following people
a1 <- 18

#minimum year of birth from which we start following people
b1 <- 1987

#Prepare sets (one for each combination of year and cohort)
#function 
prep <- function(y,c,data){
  
  a <- df %>% filter(start<b1+a1+y+c+2,birth>=b1+c,birth<b1+c+2,stop-birth>=a1) 
  a <- a %>% group_by(id) %>% mutate(index=row_number())
  a <- a %>% select(id,State,index)
  a <- a %>% spread(index,State)
  return(as.matrix(a[,2:ncol(a)]))
  
}

#run function 
#Structure of resulting data is set[[max age]][[birth year]]
#max age = 20,22,...,42   birth year = 1987,1989,...,1999
set <- lapply(seq(0,12,2), function(x) lapply(seq(0,12,2),function(z) prep(x,z,df)))

#calculate number of states
ns <- unlist(lapply(1:7,function(x) lapply(1:7, function(y) mean(rowSums(!is.na(set[[x]][[y]]))))))

#calculate number of observations
n <- unlist(lapply(1:7,function(x) lapply(1:7, function(y) length(set[[x]][[y]][,1]))))

#decomposition by state: NDS
#define function
fct.nds <- function(year,cohort,state){
  
  a <- rowSums(set[[year]][[cohort]]==state,na.rm=TRUE)
  length(a[a!=0])/length(a)
  
}

#calculation 
nds.a <- unlist(lapply(1:7,function(x) (lapply(1:7, function(y) fct.nds(x,y,"Alone")))))
nds.c <- unlist(lapply(1:7,function(x) (lapply(1:7, function(y) fct.nds(x,y,"Cohabiting")))))
nds.m <- unlist(lapply(1:7,function(x) (lapply(1:7, function(y) fct.nds(x,y,"Married")))))

#decomposition by state: ANV
#define function
fct.anv <- function(year,cohort,state){
  
  mean(rowSums(set[[year]][[cohort]]==state,na.rm=TRUE)/rowSums(!is.na(set[[year]][[cohort]])))
  
}

#calculation 
anv.a <- unlist(lapply(1:7,function(x) (lapply(1:7, function(y) fct.anv(x,y,"Alone")))))
anv.c <- unlist(lapply(1:7,function(x) (lapply(1:7, function(y) fct.anv(x,y,"Cohabiting")))))
anv.m <- unlist(lapply(1:7,function(x) (lapply(1:7, function(y) fct.anv(x,y,"Married")))))

#store results in data frame
results <- data.frame(cohort=seq(1987,1999,2),age=rep(seq(18,30,2),each=7),
                      nds.a=nds.a,nds.c=nds.c,nds.m=nds.m,
                      anv.a=anv.a,anv.c=anv.c,anv.m=anv.m,
                      ns=ns,n=n)

#nds (the sum of number of times in each state)
results$nds <- results$nds.a + results$nds.c + results$nds.m  

#mean number of visits to visited states (number of states divided by number of distinct states visited)
results$anv <- results$ns/ results$nds

#anv was calculated as proportion
#now calculate it in absolute terms based on total number of states occupied
results$anv.a <- results$anv.a * results$anv
results$anv.c <- results$anv.c * results$anv
results$anv.m <- results$anv.m * results$anv

#add year vector (age represents the average value; cohort and year the minimum)
results$year <- results$cohort + results$age 

#cutoff information that is out of bounds
results <- filter(results,year<2019)

#pivot longer
results <- gather(results,label,value,-cohort,-age,-n,-year)

#arrange with same cohort in subsequent rows
results <- arrange(results,cohort,age,label)

#find ANDS and AANV (differences between subsequent cells)
results <- results %>% group_by(cohort,label) %>% mutate(avalue=value-lag(value,default=0))

################################################################################
#RESULTS
  #compare cohorts 1987-1988 and 1989-1990 at age 28
  #and periods 2015-2016 and 2017-2018 at age 28
################################################################################
#Table 1: NDS and ANV, cohort and period
results %>% filter(age==28,label=="nds"|label=="anv") %>% 
  group_by(cohort,label) %>% summarise(value=round(sum(value),2)) %>% spread(cohort,value)

results %>% group_by(year,label) %>% 
  filter(age<=28,label=="nds"|label=="anv",year>=2015) %>% 
  summarise(value=round(sum(avalue),2)) %>% spread(year,value)

################################################################################
#Table 2: Decomposition by age
################################################################################
#cohort#########################################################################
#nds
results %>% filter(label=="nds",age<=28,cohort<=1989) %>%
  group_by(age,cohort) %>% summarise(value=round(sum(avalue),2)) %>% spread(cohort,value)

#anv
results %>% filter(label=="anv",age<=28,cohort<=1989) %>%
  group_by(age,cohort) %>% summarise(value=round(sum(avalue),2)) %>% spread(cohort,value)

#period#########################################################################
#nds
results %>% filter(label=="nds",age<=28,year>=2015) %>%
  group_by(age,year) %>% summarise(value=round(sum(avalue),2)) %>% spread(year,value)

#anv
results %>% filter(label=="anv",age<=28,year>=2015) %>%
  group_by(age,year) %>% summarise(value=round(sum(avalue),2)) %>% spread(year,value)

################################################################################
#Table 3: decomposition by state
################################################################################
#Cohorts
#nds
results %>% filter(age==28,cohort<=1989,(label=="nds.a"|label=="nds.c"|label=="nds.m")) %>%
  group_by(cohort,label) %>% summarise(value=round(sum(value),4)) %>% spread(cohort,value)

#anv
results %>% filter(age==28,cohort<=1989,(label=="anv.a"|label=="anv.c"|label=="anv.m")) %>%
  group_by(cohort,label) %>% summarise(value=round(sum(value),4)) %>% spread(cohort,value)

#Periods
#nds
results %>% filter(age==28,year>=2015,(label=="nds.a"|label=="nds.c"|label=="nds.m")) %>%
  group_by(year,label) %>% summarise(value=round(sum(value),4)) %>% spread(year,value)

#anv
results %>% filter(age==28,year>=2015,(label=="anv.a"|label=="anv.c"|label=="anv.m")) %>%
  group_by(year,label) %>% summarise(value=round(sum(value),4)) %>% spread(year,value)

################################################################################
#Lexis surfaces (Supplementary Materials)
################################################################################
#add information for drawing lines
results$agestart <- results$age-2
results$yearstop <- results$year+2

#Round
results$avalue.round <- round(results$avalue,2)

#plot nds
ggplot(filter(results,label=="nds"))+
  geom_segment(aes(x=year,xend=yearstop,y=agestart,yend=age))+
  geom_segment(aes(x=year,xend=year,y=agestart,yend=age))+
  geom_segment(aes(x=2005,xend=2019,y=18,yend=32))+
  geom_segment(aes(x=2019,xend=2019,y=18,yend=32))+
  geom_text(aes(x=year+1,y=age,label=avalue.round))+
  theme_classic()+
  ylab("Age")+
  xlab("Year")+
  scale_y_continuous(breaks=seq(16,32,2))+
  scale_x_continuous(breaks=seq(2005,2019,2))

#plot anv
ggplot(filter(results,label=="anv"))+
  geom_segment(aes(x=year,xend=yearstop,y=agestart,yend=age))+
  geom_segment(aes(x=year,xend=year,y=agestart,yend=age))+
  geom_segment(aes(x=2005,xend=2019,y=18,yend=32))+
  geom_segment(aes(x=2019,xend=2019,y=18,yend=32))+
  geom_text(aes(x=year+1,y=age,label=avalue.round))+
  theme_classic()+
  ylab("Age")+
  xlab("Year")+
  scale_y_continuous(breaks=seq(16,32,2))+
  scale_x_continuous(breaks=seq(2005,2019,2))

